// ===============================================================
// ================== SOFTWARE INSTALLAZIONE =====================
// ==================    Robot Caotici 2.1   =====================
// ===============================================================

#include <OrangutanAnalog.h>
#include <OrangutanMotors.h>
OrangutanAnalog analog;
OrangutanMotors motors;

const int Num = 4;  // Numero letture ADC per media

// ========================================================================
// ============================= PIN SENSORI ==============================
// ========================================================================
const int sharpL_pin = 1;   // SHARP SINISTRO
const int sharpR_pin = 2;   // SHARP DESTRO
const int battery_pin = 5;  // Batteria
const int cny70L_pin = 3;   // CNY70 SINISTRO
const int cny70R_pin = 4;   // CNY70 DESTRO

float sVL = 1;
float sVR = 1;

// 1 = bordo || 0 = campo
const int B = 0;   // Bianco
const int N = 1;   // Nero
const int L = -1;  // Left
const int R = 1;   // Right

const int Vmax0 = 150; //100
const int Vmed0 = 82;  //55
const int Vmin0 = 52;  //35
int Vmax = Vmax0;
int Vmed = Vmed0;
int Vmin = Vmin0;
int VL = Vmed0;
int VR = Vmed0;

const int dist = 100;   // Distanza minima
const int max_battery = 800; // Valore letto dall'ADC a batterie nuove
const int soglia = 500;   // soglia B-N

int val_cny70L = B;
int val_cny70R = B;
int val_cny70L_pre = B;
int val_cny70R_pre = B;
int Aval_cny70L = 0;
int Aval_cny70R = 0;
int val_sharpL = 0;
int val_sharpR = 0;
int val_battery = 0;

int dir = 0;

unsigned long now, nextBatt, nextMap;

float Zk = 0.3;

void setup() {
  analog.setMode(MODE_10_BIT);    // 10-bit analog-to-digital conversions  
  nextBatt = millis();
  int pot = analog.readTrimpot();    // determine the trimpot position
  /*
  if (pot >=255) 
  {
    sVL=1+(pot/100);
    sVR=1;
  }
  else
  {
    sVL=1;
    sVR=1+(pot/100);
  }
  */
  //Serial.begin(7680);
}

void loop()
{
  // ========================================================================
  // ======================= CONTROLLO CARICA BATTERIA ======================
  // ========================================================================
  now = millis();
  if (now > nextBatt) {
    nextBatt = now + 60000;

    val_battery = 0;
    for (int j = 0; j < Num; j++) {
      val_battery += analog.read(battery_pin);
      delayMicroseconds(100);
    }
    val_battery /= Num;  

    Vmin = float(Vmin0) * float(max_battery) / float(val_battery);
    Vmed = float(Vmed0) * float(max_battery) / float(val_battery);
    Vmax = float(Vmax0) * float(max_battery) / float(val_battery);

    /*Serial.print("Vmed = ");
     Serial.print(Vmed);
     Serial.print(" Vme = ");
     Serial.print(Vme);
     Serial.print(" Vmax = ");
     Serial.print(Vmax);
     Serial.print(" Vma = ");
     Serial.println(Vma);*/
  }

  // Mappa Logistica
  now = millis();
  if (now > nextMap) {
    nextMap = now + 3000;

    Zk = 4.0*Zk*(1 - Zk);

    if (Zk < 0.4){
      VL = Vmed;
      VR = Vmin;
    }
    else if (Zk < 0.65) {
      VL = Vmed;
      VR = Vmed;
    }
    else {
      VL = Vmin;
      VR = Vmed;
    }
    
    //VL = Vmed;
    //VR = Vmed;
  }

  // Lettura valori dai sensori
  val_sharpL = val_sharpR = Aval_cny70L = Aval_cny70R = 0;
  for (int j = 0; j < Num; j++) {
    val_sharpL += analog.read(sharpL_pin);
    delayMicroseconds(100);
    val_sharpR += analog.read(sharpR_pin);
    delayMicroseconds(100);
    Aval_cny70L += analog.read(cny70L_pin);
    delayMicroseconds(100);
    Aval_cny70R += analog.read(cny70R_pin);
    delayMicroseconds(100);
  }
  val_sharpL /= Num;
  val_sharpR /= Num;
  Aval_cny70L /= Num;
  Aval_cny70R /= Num;
  
/*
  Serial.print("cnyL = ");
  Serial.println(Aval_cny70L);
  Serial.print("cnyR = ");
  Serial.println(Aval_cny70R);
*/

  // ========================================================================
  // ==========  conversione soglia bianco/vero cny70 su analogico ==========
  // ========================================================================
  if (Aval_cny70L < soglia){
    val_cny70L=B;
  } //[>]bianco[<]nero campo
  else {
    val_cny70L=N;
  }
  if (Aval_cny70R < soglia){
    val_cny70R=B;
  } //[>]bianco[<]nero campo
  else {
    val_cny70R=N;
  }

  // =============================================================
  // =========== CNY70 programmazione a stati finiti =============
  // =============================================================

  if (val_cny70L == B && val_cny70R == B && val_cny70L_pre == B && val_cny70R_pre == B) { 
    motors.setSpeeds(VL*sVL, VR*sVR);  // set speeds of motors 1 (SX) and 2 (DX)
  }
  else if (val_cny70L == N && val_cny70R == N && val_cny70L_pre == B && val_cny70R_pre == B) { 
    motors.setSpeeds(Vmed*sVL, -Vmed*sVR);  // set speeds of motors 1 (SX) and 2 (DX)
    dir = R;
  }
  else if (val_cny70L == N && val_cny70R == B && val_cny70L_pre == B && val_cny70R_pre == B) { 
    motors.setSpeeds(Vmed*sVL, -Vmed*sVR);  // set speeds of motors 1 (SX) and 2 (DX)
  } 
  else if (val_cny70L == N && val_cny70R == B && val_cny70L_pre == N && val_cny70R_pre == B) { 
    motors.setSpeeds(Vmed*sVL, -Vmed*sVR);  // set speeds of motors 1 (SX) and 2 (DX)
  }
  else if (val_cny70L == B && val_cny70R == B && val_cny70L_pre == N && val_cny70R_pre == B) { 
    motors.setSpeeds(VL*sVL, VR*sVR);  // set speeds of motors 1 (SX) and 2 (DX) 
  }
  else if (val_cny70L == N && val_cny70R == N && val_cny70L_pre == N && val_cny70R_pre == B) { 
    motors.setSpeeds(Vmed*sVL, -Vmed*sVR);  // set speeds of motors 1 (SX) and 2 (DX)
    dir = R;
  }
  else if (val_cny70L == B && val_cny70R == N && val_cny70L_pre == B && val_cny70R_pre == B) { 
    motors.setSpeeds(-Vmed*sVL, Vmed*sVR);  // set speeds of motors 1 (SX) and 2 (DX) 
  }
  else if (val_cny70L == B && val_cny70R == N && val_cny70L_pre == B && val_cny70R_pre == N) { 
    motors.setSpeeds(-Vmed*sVL, Vmed*sVR);  // set speeds of motors 1 (SX) and 2 (DX) 
  }
  else if (val_cny70L == B && val_cny70R == B && val_cny70L_pre == B && val_cny70R_pre == N) { 
    motors.setSpeeds(VL*sVL, VR*sVR);  // set speeds of motors 1 (SX) and 2 (DX) 
  }
  else if (val_cny70L == N && val_cny70R == N && val_cny70L_pre == B && val_cny70R_pre == N) { 
    motors.setSpeeds(-Vmed*sVL, Vmed*sVR);  // set speeds of motors 1 (SX) and 2 (DX) 
    dir = L;
  }
  else if (val_cny70L == N && val_cny70R == N && val_cny70L_pre == N && val_cny70R_pre == N) {
    if (dir == L) {
      motors.setSpeeds(-Vmed*sVL, Vmed*sVR);  // set speeds of motors 1 (SX) and 2 (DX) 
    } 
    else if (dir == R) {
      motors.setSpeeds(Vmed*sVL, -Vmed*sVR);  // set speeds of motors 1 (SX) and 2 (DX)
    }
  }
  else if (val_cny70L == B && val_cny70R == N && val_cny70L_pre == N && val_cny70R_pre == N) { 
    if (dir == L) {
      motors.setSpeeds(-Vmed*sVL, Vmed*sVR);  // set speeds of motors 1 (SX) and 2 (DX) 
    } 
    else if (dir == R) {
      motors.setSpeeds(Vmed*sVL, -Vmed*sVR);  // set speeds of motors 1 (SX) and 2 (DX)
    }
  }
  else if (val_cny70L == N && val_cny70R == B && val_cny70L_pre == N && val_cny70R_pre == N) { 
    if (dir == L) {
      motors.setSpeeds(-Vmed*sVL, Vmed*sVR);  // set speeds of motors 1 (SX) and 2 (DX) 
    } 
    else if (dir == R) {
      motors.setSpeeds(Vmed*sVL, -Vmed*sVR);  // set speeds of motors 1 (SX) and 2 (DX)
    }
  }
  else if (val_cny70L == B && val_cny70R == B && val_cny70L_pre == N && val_cny70R_pre == N) { 
    motors.setSpeeds(VL*sVL, VR*sVR);  // set speeds of motors 1 (SX) and 2 (DX) 
  }
  val_cny70L_pre = val_cny70L;
  val_cny70R_pre = val_cny70R;

  // ===========================================
  // ========== SHARP anti-collisione ==========
  // ===========================================

  if (val_cny70L == B && val_cny70R == B) {
    if (val_sharpL <= dist && val_sharpR <= dist) {
    }
    else {
      if (val_sharpR > dist && val_sharpR >= val_sharpL) {
        motors.setSpeeds(-Vmed*sVL, Vmed*sVR);  // set speeds of motors 1 (SX) and 2 (DX) 
      }
      if (val_sharpL > dist && val_sharpR <= val_sharpL) {
        motors.setSpeeds(Vmed*sVL, -Vmed*sVR);  // set speeds of motors 1 (SX) and 2 (DX) 
      }
    }
  }
  delay(1);
}

